

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/* ficher probecst.c
   en passant par une fifo intermediaire
   ainsi les proc atl_dump_inst() et atl_dump_all_inst() sont utilises
*/
#include <memory.h>

#ifdef USE_ATS
#undef USE_ATS
#endif
#define USE_ATS 1

#include "TP.h"
#include "probecst.h"
#define trace printf

atl_declar;

static void fifo_in (s_fifo *fifo,void *el,int size) 
{
  
  int tmp = fifo->queue+size;
  while (tmp >= fifo->size)
    fifo->size+=SIZE_BLOCK_LIFO;
  
  fifo->data=(char *)atl_realloc(fifo->data,fifo->size);
  memcpy(fifo->data+fifo->queue,el,size);
  fifo->queue+=size;
}

static void * fifo_out (s_fifo *fifo,int size)
{
  void *tmp=fifo->data+fifo->head;
  fifo->head+=size;
  return tmp;
}

static int fifo_empty (s_fifo *fifo)
{
  return (fifo->head == fifo->queue ? 1 : 0 );
}

static void fifo_init (s_fifo *fifo)
{
  fifo->data=(char *)atl_malloc(SIZE_BLOCK_LIFO*sizeof(char)); 
  fifo->size=SIZE_BLOCK_LIFO;
  fifo->head=0;
  fifo->queue=0;
}

static void fifo_close (s_fifo *fifo)
{
  atl_free(fifo->data);
}

EXTERN void atl_dump_inst (ATL_CTX_PRB prb_ctx,ATL_PT_CAR path,ATL_PT_CAR riofile,ATL_PT_CAR instName,ATL_T_UINT other);

/****************************************************************************
 *.DESCRIPTION : This routine create FIFO for application
 *.INPUT : 
 *		- size  		nb Bytes of the fifo
 *		- path			path given by the macro start
 *		- instance		instance name given in the macro start
 *		- size			fifo size given in the macro start
 *.OUTPUT :
 *		- return		fifo ptr
 ***************************************************************************/
ATL_CTX_PRB atl_create_probe(char *path,char *instance,int size)
{
  ATL_CTX_PRB fifo=(ATL_CTX_PRB)atl_malloc(sizeof(s_fifo)); 
  fifo_init(fifo);
  
 /* at this point we may create thread atl_dump_inst to read fifo in backGround */
 return fifo;
}

/****************************************************************************
 *.DESCRIPTION : This routine mark the end of data write into selected FIFO
 *.INPUT : 
 *		- fifo		    fifo ptr
 ***************************************************************************/
void atl_end_probe(ATL_CTX_PRB ctx)
{
  /* empty */
}

/****************************************************************************
 *.DESCRIPTION : This routine write data into selected FIFO for application
 *					record is:lenMsg,time,code,msg.
 *.INPUT : 
 *		- fifo		    fifo ptr to put the buffer
 *		- code			TDC code
 *		- key			data not used spare for union.select
 *		- msg			address where the message is
 *		- lenmsg		message length
 ***************************************************************************/
void atl_write_probe(ATL_CTX_PRB fifo,unsigned int code, unsigned int key, void *msg, unsigned int lenmsg)
{
  ATL_T_DATE time;
  atl_get_date(&time);
  
  fifo_in(fifo,&lenmsg,sizeof(unsigned int));
  fifo_in(fifo,&time,sizeof(ATL_T_DATE));
  fifo_in(fifo,&code,sizeof(unsigned int));
  fifo_in(fifo,msg,lenmsg);
}


/****************************************************************************
 *	.DESCRIPTION : This routine read data into selected FIFO for application
 *	.INPUT : 
 *			- fifo		    fifo ptr to put the buffer
 *	.OUTPUT :
 *			- code			TDC code
 *			- key			data not used spare for union.select
 *			- msg			address where the message is
 *			- return		1 if record exist, 0 if end
 ***************************************************************************/
int atl_read_probe(ATL_CTX_PRB fifo, ATL_T_DATE *time, unsigned int *code, unsigned int *key, void **msg)
{
  int lenmsg=0;  
  char *tmp=(char *)0;
  if (fifo_empty(fifo))
    return 0;
  atl_memcpy(&lenmsg,fifo_out(fifo,sizeof(unsigned int)),sizeof(unsigned int));  
  atl_memcpy(time,fifo_out(fifo,sizeof(ATL_T_DATE)),sizeof(ATL_T_DATE));
  atl_memcpy(code,fifo_out(fifo,sizeof(unsigned int)),sizeof(unsigned int));
  tmp=(char *)fifo_out(fifo,lenmsg);
  *msg=(char *)atl_malloc(lenmsg);
  atl_memcpy(*msg,tmp,lenmsg);
  return  1;    
}

/****************************************************************************
 *	.DESCRIPTION : This routine is called at fifo dump start 
 *	.INPUT : 
 *			- prb_ctx		fifo ptr where messages are written
 *			- path			path given by the macro start
 *			- riofile		rio file name given in the macro start
 *			- instance		instance name given in the macro start
 *			- p1			fifo size given in the macro start
 *	.OUTPUT :
 *			- return		fifo ptr
 ***************************************************************************/
ATL_CTX_PRB atl_open_probe(ATL_CTX_PRB fifo, char *path,char *riofile, char *instance,int p1)
{
  return fifo;
}

/****************************************************************************
 *	.DESCRIPTION : This routine is called at the end of the fifo dump
 *	.INPUT : 
 *			- ctx		    fifo ptr 
 ***************************************************************************/
void atl_close_probe(ATL_CTX_PRB fifo)
{
  fifo_close(fifo);
}


void atl_write_key(ATL_CTX_PRB ctx,unsigned int code,unsigned int value)
{
  unsigned int msg;;
  msg=value;
  atl_write_probe(ctx,code,0,&msg,sizeof(unsigned int));
}



